--have to override this function completely to support all the changes to build custom weapons from charactertweakdata
function MenuSceneManager:set_henchmen_loadout(index, character, loadout)
	self._picked_character_position = self._picked_character_position or {}
	loadout = loadout or managers.blackmarket:henchman_loadout(index)
	character = character or managers.blackmarket:preferred_henchmen(index)

	if not character then
		local preferred = managers.blackmarket:preferred_henchmen()
		local characters = CriminalsManager.character_names()
		local player_character = managers.blackmarket:get_preferred_characters_list()[1]
		local available = {}
		local t_get_key = table.get_key
		local t_contains = table.contains

		for i = 1, #characters do
			local name = characters[i]

			if player_character ~= name then
				local found_current = t_get_key(self._picked_character_position, name) or 999

				if not t_contains(preferred, name) and index <= found_current then
					local new_name = CriminalsManager.convert_old_to_new_character_workname(name)
					local char_tweak = tweak_data.blackmarket.characters.locked[new_name] or tweak_data.blackmarket.characters[new_name]

					if not char_tweak.dlc or managers.dlc:is_dlc_unlocked(char_tweak.dlc) then
						available[#available + 1] = name
					end
				end
			end
		end

		if #available < 1 then
			available = CriminalsManager.character_names()
		end

		character = available[math.random(#available)] or "russian"
	end

	self._picked_character_position[index] = character
	local character_id = managers.blackmarket:get_character_id_by_character_name(character)
	local unit = self._henchmen_characters[index]

	self:_delete_character_weapon(unit, "all")

	local unit_name = tweak_data.blackmarket.characters[character_id].menu_unit
	local was_alive = alive(unit)

	if not was_alive or Idstring(unit_name) ~= unit:name() then
		local pos, rot = nil

		if was_alive then
			pos = unit:position()
			rot = unit:rotation()

			self:_delete_character_mask(unit)
			World:delete_unit(unit)
		end

		unit = World:spawn_unit(Idstring(unit_name), pos or Vector3(), rot or Rotation())

		self:_init_character(unit, index)

		self._henchmen_characters[index] = unit
	end

	unit:base():set_character_name(character)

	local mask = loadout.mask
	local mask_blueprint = loadout.mask_blueprint
	local crafted_mask = managers.blackmarket:get_crafted_category_slot("masks", loadout.mask_slot)

	if crafted_mask then
		mask = crafted_mask.mask_id
		mask_blueprint = crafted_mask.blueprint
	end

	self:set_character_mask_by_id(mask, mask_blueprint, unit, nil, character)

	local mask_data = self._mask_units[unit:key()]

	if mask_data then
		self:update_mask_offset(mask_data)
	end

	local weapon_id = nil
	local crafted_primary = managers.blackmarket:get_crafted_category_slot("primaries", loadout.primary_slot)

	if crafted_primary then
		local primary = crafted_primary.factory_id
		local primary_id = crafted_primary.weapon_id
		local primary_blueprint = crafted_primary.blueprint
		local primary_cosmetics = crafted_primary.cosmetics

		self:set_character_equipped_weapon(unit, primary, primary_blueprint, "primary", primary_cosmetics)

		weapon_id = primary_id
	else
		local primary = tweak_data.character[character].weapon.weapons_of_choice.primary
		local blueprint, cosmetics = nil

		if type(primary) == "table" then
			primary = primary[math.random(#primary)]

			blueprint = primary.blueprint
			cosmetics = primary.cosmetics
			primary = primary.factory_name
			primary = string.gsub(primary, "_npc", "")

			if not blueprint or not next(blueprint) then
				blueprint = managers.weapon_factory:get_default_blueprint_by_factory_id(primary)
			else
				local factory_manager = managers.weapon_factory
				local modified_default = clone(factory_manager:get_default_blueprint_by_factory_id(primary))
				local part_data_f = factory_manager._part_data

				for i = 1, #blueprint do
					local part_id = blueprint[i]
					local part_data = part_data_f(factory_manager, part_id, primary)

					if part_data then
						for idx = 1, #modified_default do
							local default_part_id = modified_default[idx]
							local default_part_data = part_data_f(factory_manager, default_part_id, primary)

							if default_part_data then
								if part_data.type == default_part_data.type then
									local new_default_blueprint = {}

									for old_i = 1, idx - 1 do
										new_default_blueprint[#new_default_blueprint + 1] = modified_default[old_i]
									end

									for old_i = idx + 1, #modified_default do
										new_default_blueprint[#new_default_blueprint + 1] = modified_default[old_i]
									end

									modified_default = new_default_blueprint

									break
								end
							end
						end
					end
				end

				for i = 1, #blueprint do
					local part_id = blueprint[i]

					modified_default[#modified_default + 1] = part_id
				end

				blueprint = modified_default
			end

			if cosmetics and not cosmetics.id then
				cosmetics = nil
			end
		else
			primary = string.gsub(primary, "_npc", "")

			blueprint = managers.weapon_factory:get_default_blueprint_by_factory_id(primary)
		end

		self:set_character_equipped_weapon(unit, primary, blueprint, "primary", cosmetics)

		weapon_id = managers.weapon_factory:get_weapon_id_by_factory_id(primary)
	end

	self:set_character_player_style(loadout.player_style, loadout.suit_variation, unit)
	self:set_character_gloves(loadout.glove_id, unit)
	self:_select_henchmen_pose(unit, weapon_id, index)

	local pos, rot = self:get_henchmen_positioning(index)

	unit:set_position(pos)
	unit:set_rotation(rot)
	self:set_henchmen_visible(true, index)
end
