--custom weapon unit spawn function (compared to CopMovement)
--relies on TeamAIBase having its own melee_weapon() function to work
function TeamAIMovement:anim_clbk_enemy_spawn_melee_item()
	if alive(self._melee_item_unit) then
		return
	end

	local unit_name = self._ext_base:melee_weapon()

	if not unit_name then
		return
	end

	local unit = self._unit
	local align_obj_l_name = CopMovement._gadgets.aligns.hand_l
	local align_obj_l = unit:get_object(align_obj_l_name)
	local melee_unit = World:spawn_unit(unit_name, align_obj_l:position(), align_obj_l:rotation())
	self._melee_item_unit = melee_unit

	unit:link(align_obj_l:name(), melee_unit, melee_unit:orientation_object():name())
end

--overriding to fix some vanilla issues because the function doesn't execute inhereted pre_destroy code and because otherwise the post hook there wouldn't be used
function TeamAIMovement:pre_destroy()
	TeamAIMovement.super.pre_destroy(self)

	if self._heat_listener_clbk then
		managers.groupai:state():remove_listener(self._heat_listener_clbk)

		self._heat_listener_clbk = nil
	end

	if self._switch_to_not_cool_clbk_id then
		managers.enemy:remove_delayed_clbk(self._switch_to_not_cool_clbk_id)

		self._switch_to_not_cool_clbk_id = nil
	end
end

TeamAIMovement.selected_primaries = {}
TeamAIMovement.selected_secondaries = {}

--have to override this function completely to support all the changes to build custom weapons from charactertweakdata
function TeamAIMovement:add_weapons()
	--if this somehow happens
	if not Network:is_server() then
		HuskTeamAIMovement.add_weapons(self)

		return
	end

	local char_name = self._ext_base._tweak_table
	local loadout = managers.criminals:get_loadout_for(char_name)

	local crafted_primary = managers.blackmarket:get_crafted_category_slot("primaries", loadout.primary_slot)
	local primary_weapon = self:chk_spawn_custom_default_weapon("primary", char_name, loadout, crafted_primary)

	local crafted_secondary = managers.blackmarket:get_crafted_category_slot("secondaries", loadout.secondary_slot)
	self:chk_spawn_custom_default_weapon("secondary", char_name, loadout, crafted_secondary, primary_weapon)
end

--custom function to spawn crafted default weapons for the team AI, defined in charactertweakdata
function TeamAIMovement:chk_spawn_custom_default_weapon(slot, char_name, loadout, crafted, other_weapon)
	local base_ext = self._ext_base
	local inv_ext = self._ext_inventory
	local slotted_loadout = loadout[slot]

	if crafted then --modified weapon equipped by the player on the menu
		inv_ext:add_unit_by_factory_blueprint(slotted_loadout, false, false, crafted.blueprint, crafted.cosmetics)
	elseif slotted_loadout then --unmodified weapon equipped by the player on the menu
		inv_ext:add_unit_by_factory_name(slotted_loadout, false, false, nil, "")
	else
		local weapon = base_ext:default_weapon_name(slot) --get weapon name from charactertweakdata
		local blueprint, cosmetics = nil

		--if the name is a table instead of a string, then it's a custom default weapon
		if type(weapon) == "table" then
			--if a bot already spawned with this method, check for the existing entry to avoid repeating all the steps
			local already_selected_table = slot == "primary" and TeamAIMovement.selected_primaries or TeamAIMovement.selected_secondaries
			local already_selected = already_selected_table[char_name]

			if already_selected then
				weapon = already_selected[1]
				blueprint = already_selected[2]
				cosmetics = already_selected[3]
			else
				--in case more than one weapon was defined in this table, randomize which one is selected
				--the system intends you to use a table even with just one weapon
				weapon = weapon[math.random(#weapon)]

				blueprint = weapon.blueprint
				cosmetics = weapon.cosmetics
				weapon = weapon.factory_name

				--if no blueprint table was defined, or if its empty, get the default blueprint for the weapon
				if not blueprint or not next(blueprint) then
					blueprint = managers.weapon_factory:get_default_blueprint_by_factory_id(weapon)
				else
					--basically ensure that the parts that are being added are compatible with the weapon
					local factory_manager = managers.weapon_factory
					local modified_default = clone(factory_manager:get_default_blueprint_by_factory_id(weapon))
					local part_data_f = factory_manager._part_data

					for i = 1, #blueprint do
						local part_id = blueprint[i]
						local part_data = part_data_f(factory_manager, part_id, weapon)

						if part_data then
							for idx = 1, #modified_default do
								local default_part_id = modified_default[idx]
								local default_part_data = part_data_f(factory_manager, default_part_id, weapon)

								if default_part_data then
									if part_data.type == default_part_data.type then
										local new_default_blueprint = {}

										for old_i = 1, idx - 1 do
											new_default_blueprint[#new_default_blueprint + 1] = modified_default[old_i]
										end

										for old_i = idx + 1, #modified_default do
											new_default_blueprint[#new_default_blueprint + 1] = modified_default[old_i]
										end

										modified_default = new_default_blueprint

										break
									end
								end
							end
						end
					end

					for i = 1, #blueprint do
						local part_id = blueprint[i]

						modified_default[#modified_default + 1] = part_id
					end

					blueprint = modified_default
				end

				--if the cosmetics table lacks an id, remove it
				if cosmetics and not cosmetics.id then
					cosmetics = nil
				end

				already_selected_table[char_name] = {
					[1] = weapon,
					[2] = blueprint,
					[3] = cosmetics
				}
			end
		end

		if weapon then
			--if primary, proceed as usual, if secondary, check that the it's not exactly the same as the primary, just in case
			if not other_weapon or weapon ~= other_weapon then
				local factory_entry = tweak_data.weapon.factory[weapon]

				if not factory_entry then
					log("[TeamAIMovement:chk_spawn_custom_default_weapon] ERROR - no valid factory entry for this weapon, possible incompatibility from another mod")

					return
				end

				if blueprint then
					inv_ext:add_unit_by_factory_blueprint(weapon, false, false, blueprint, cosmetics)
				else
					inv_ext:add_unit_by_factory_name(weapon, false, false, nil, "")
				end
			end

			return weapon
		end
	end
end
